@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2013-2023, The PurpleI2P Project
+* Copyright (c) 2013-2024, The PurpleI2P Project
 *
 * This file is part of Purple i2pd project and licensed under BSD3
 *
@@ -15,7 +15,6 @@
 #include <condition_variable>
 #include <openssl/rand.h>
 #include <boost/algorithm/string.hpp>
-#include <boost/filesystem.hpp>
 #include "Base.h"
 #include "util.h"
 #include "Identity.h"
@@ -27,6 +26,14 @@
 #include "AddressBook.h"
 #include "Config.h"
 
+#if STD_FILESYSTEM
+#include <filesystem>
+namespace fs_lib = std::filesystem;
+#else
+#include <boost/filesystem.hpp>
+namespace fs_lib = boost::filesystem;
+#endif
+
 namespace i2p
 {
 namespace client
@@ -266,11 +273,11 @@ namespace client
 	void AddressBookFilesystemStorage::ResetEtags ()
 	{
 		LogPrint (eLogError, "Addressbook: Resetting eTags");
-		for (boost::filesystem::directory_iterator it (etagsPath); it != boost::filesystem::directory_iterator (); ++it)
+		for (fs_lib::directory_iterator it (etagsPath); it != fs_lib::directory_iterator (); ++it)
 		{
-			if (!boost::filesystem::is_regular_file (it->status ()))
+			if (!fs_lib::is_regular_file (it->status ()))
 				continue;
-			boost::filesystem::remove (it->path ());
+			fs_lib::remove (it->path ());
 		}
 	}
 
@@ -434,7 +441,7 @@ namespace client
 			auto ident = std::make_shared<i2p::data::IdentityEx>();
 			if (ident->FromBase64 (jump))
 			{
-				m_Storage->AddAddress (ident);
+				if (m_Storage) m_Storage->AddAddress (ident);
 				m_Addresses[address] = std::make_shared<Address>(ident->GetIdentHash ());
 				LogPrint (eLogInfo, "Addressbook: Added ", address," -> ", ToAddress(ident->GetIdentHash ()));
 			}
@@ -445,18 +452,19 @@ namespace client
 
 	void AddressBook::InsertFullAddress (std::shared_ptr<const i2p::data::IdentityEx> address)
 	{
-		m_Storage->AddAddress (address);
+		if (m_Storage) m_Storage->AddAddress (address);
 	}
 
 	std::shared_ptr<const i2p::data::IdentityEx> AddressBook::GetFullAddress (const std::string& address)
 	{
 		auto addr = GetAddress (address);
 		if (!addr || !addr->IsIdentHash ()) return nullptr;
-		return m_Storage->GetAddress (addr->identHash);
+		return m_Storage ? m_Storage->GetAddress (addr->identHash) : nullptr;
 	}
 
 	void AddressBook::LoadHosts ()
 	{
+		if (!m_Storage) return;
 		if (m_Storage->Load (m_Addresses) > 0)
 		{
 			m_IsLoaded = true;
@@ -527,15 +535,18 @@ namespace client
 						ident->GetSigningKeyType () != i2p::data::SIGNING_KEY_TYPE_DSA_SHA1) // don't replace by DSA
 					{
 						it->second->identHash = ident->GetIdentHash ();
-						m_Storage->AddAddress (ident);
-						m_Storage->RemoveAddress (it->second->identHash);
+						if (m_Storage)
+						{	
+							m_Storage->AddAddress (ident);
+							m_Storage->RemoveAddress (it->second->identHash);
+						}	
 						LogPrint (eLogInfo, "Addressbook: Updated host: ", name);
 					}
 				}
 				else
 				{
 					m_Addresses.emplace (name, std::make_shared<Address>(ident->GetIdentHash ()));
-					m_Storage->AddAddress (ident);
+					if (m_Storage) m_Storage->AddAddress (ident);
 					if (is_update)
 						LogPrint (eLogInfo, "Addressbook: Added new host: ", name);
 				}
@@ -547,7 +558,7 @@ namespace client
 		if (numAddresses > 0)
 		{
 			if (!incomplete) m_IsLoaded = true;
-			m_Storage->Save (m_Addresses);
+			if (m_Storage) m_Storage->Save (m_Addresses);
 		}
 		return !incomplete;
 	}
